---- DATA STRUCTURES ----
--[[
    PlaySound = 
    {
        scope = "",
        name = "",
        x = 0,
        y = 0,
        z = 0,
        onPlayer = false,
        playWhileInside = true,
        attractZombies = false,
        hearableAddMaxDistance = Constants.HearableAddMaxDistance,
    }    
--]]

---- IMPORTS ----
local Constants = require "SoundManager/Monkey_SoundManagerConstants";
local SoundList = require 'SoundManager/Monkey_SoundList';
local Monkey_Logger = require 'Logging/Monkey_Logger';
local Log = Monkey_Logger:new('Monkey_ClientSoundManager');

---- INITIALISATION CHECK ----

if not isServer() then
    Log:LogInfo('Initialised Monkey_ClientSoundManager!');
else
    return;
end

---- PRIVATE FUNCTIONS ----

local function playSound(scope, name, x, y, z, attractZombies)
    local sound = SoundList[scope][name];
    if sound then
        sound:play(x, y, z, attractZombies);
    end
end

---- SERVER COMMAND INTEGRATION ----

local function onServerCommand(scopeName, command, args)
	if scopeName ~= Constants.ClientSoundManagerScope then 
        return 
    end;

    if args == nil then
        return;
    end

    --Handle Command
    if command == Constants.PlaySoundCommand then
        local playerPos = {}
        local player = getPlayer();
        local sq = player:getCurrentSquare();
        playerPos.x = sq:getX();
        playerPos.y = sq:getY();

        if args.playWhileInside ~= nil and args.playWhileInside == false then
            if not Monkey_Utils:IsOutside(player) then
                return;
            end
        end

        if not args.onPlayer then
            local distance = Monkey_Utils:getDistanceBetweenXY(playerPos.x, playerPos.y, args.x, args.y);
            local sl = SoundList;
            local sound = SoundList[args.scope][args.name];
            args.hearableAddMaxDistance = args.hearableAddMaxDistance or Constants.HearableAddMaxDistance;

            if distance > (sound.sound.range + 0) then
                return;
            end    
        else
            args.x = playerPos.x;
            args.y = playerPos.y;
            args.z = playerPos.z or 0;
        end

        playSound(args.scope, args.name, args.x, args.y, args.z, args.attractZombies);
    end

    Log:LogInfo('Recieved ' .. scopeName .. '-' .. command .. ' command from server!');
end

local function onGameStart_initialise()
    Events.OnServerCommand.Add(onServerCommand);
end

Events.OnGameStart.Add(onGameStart_initialise);