-- COURTESY OF KONIJIMA https://github.com/Konijima --

local json = require("Libraries/Json");
local LibConstants = require "MonkeyLibrary_Constants";
local Monkey_Logger = require 'Logging/Monkey_Logger';

local Log = Monkey_Logger:new(LibConstants.FileUtilsScope);

local Monkey_File_Utils = {};

---Encode a table to a json string
---@param data table
---@return string
function Monkey_File_Utils.TableToJson(data)
    return json.Encode(data);
end

---Decode a json string to a table
---@param str string
---@return table
function Monkey_File_Utils.JsonToTable(str)
    if not str or str == '' then
        return nil;
    end

    return json.Decode(str);
end

---Write a string to a file
---@param filename string
---@param str string
function Monkey_File_Utils.WriteFile(filename, str)
    local writer = getFileWriter(filename, true, false);
    if writer then
        writer:write(str);
        writer:close();
    end

    Log:LogInfo('Wrote to file ['.. filename ..']');
end

---Read a string from a file
---@param filename string
---@return string
function Monkey_File_Utils.ReadFile(filename)
    local str = "";
    local reader = getFileReader(filename, false);
    if reader then
        while true do
            local line = reader:readLine()
            if line == nil then
                reader:close()
                break
            end
            str = str .. line;
        end
    end
    Log:LogInfo('Read file ['.. filename ..']');
    return str;
end

---Write a table to a file
---@param filename string
---@param tbl table
function Monkey_File_Utils.WriteTable(filename, tbl)
    local str = Monkey_File_Utils.TableToJson(tbl);
    Monkey_File_Utils.WriteFile(filename, str);
end

---Read a table from a file
---@param filename string
---@return table
function Monkey_File_Utils.ReadTable(filename)
    local str = Monkey_File_Utils.ReadFile(filename);
    local tbl = Monkey_File_Utils.JsonToTable(str);
    return tbl;
end

return Monkey_File_Utils;
