-- Courtesy of KONIJIMA's https://steamcommunity.com/sharedfiles/filedetails/?id=2834231099 --

---- DATA STRUCTURES ----
--[[
    sounds = 
    {
        name = "",
        range = 1000
    }    
--]]

local Sound = require 'SoundManager/Monkey_Sound';
local SoundList = require 'SoundManager/Monkey_SoundList';
local Monkey_Logger = require 'Logging/Monkey_Logger';

local Log = Monkey_Logger:new('Monkey_SoundListManager');
local SoundListManager = {};

function SoundListManager.Add(scope, sounds)
    SoundList[scope] = SoundList[scope] or {};

    if scope and #sounds then
        for i = 1, #sounds do
            local sound = sounds[i]
            if SoundList[scope][sound.name] and #SoundList[scope][sound.name] then
                Log:LogWarning("A sound with scope [" .. scope .. "] and name [" .. sound.name .. "] wasn't added because it is already found!");
                return;
            end

            SoundList[scope][sound.name] = Sound:new(sound)
        end

        Log:LogInfo("Added sounds to scope [" .. scope .. "] containing [" .. tostring(#sounds) .. " sounds]");
    else
        Log:LogInfo("Skipping adding sounds to scope [" .. scope .. "] because sound list is empty!");
    end
end

return SoundListManager;


--[[
    --Usage Example
    --You should first add the sounds to the scripts.
    --Then use their respective name in the "name" field

    local SoundListManager = require 'SoundManager/Monkey_SoundListManager';

    SoundListManager.Add("ModName", {
        { name = "Scream", range = 100 },
        { name = "GunShot", range = 500 },
        { name = "Siren", range = 2000 },
    });    
--]]